subroutine limiter(wk,wL,wR,ix,type)
!-----------------------------------------------------------------------
!     Slope limiter
!-----------------------------------------------------------------------
!     2009/08/01  S. Zenitani  2nd order limiters (minmod, MC)
!-----------------------------------------------------------------------
  implicit none
  integer :: i, ix
  real(8) :: wk(ix)          ! work array (input)
  real(8) :: wL(ix), wR(ix)  ! interpolated values (output)
  integer :: type            ! 0 (flat), 1 (minmod), 2 (MC limiter)
  real(8) :: gA, gB, gC      ! average, left, and right gradients
  real(8) :: grad

! 1st order
  if( type .eq. 0 ) then
     do i=1,ix-1
        wL(i) = wk(i)
        wR(i) = wk(i+1)
     enddo

! 2nd order
! minmod limiter
  elseif( type .eq. 1 ) then

     do i=1,1
        wL(i)   = wk(i)
     enddo
     do i=2,ix-1
        gA =     ( wk(i)  -wk(i-1))
        gB =     ( wk(i+1)-wk(i)  )
        if( gA*gB .le. 0 ) then
           grad = 0.d0
        else
           if( gA .gt. 0 ) then
              grad = min(gA,gB)
           else
              grad = max(gA,gB)
           endif
        endif
        wR(i-1) = wk(i) - 0.5d0 * grad
        wL(i)   = wk(i) + 0.5d0 * grad
     enddo
     do i=ix,ix
        wR(i-1) = wk(i)
     enddo
         
! 2nd order
! Monotonized central-difference (MC) limitter
  elseif( type .eq. 2 ) then
     
     do i=1,1
        wL(i)   = wk(i)
     enddo
     do i=2,ix-1
        gA =       ( wk(i)  -wk(i-1))
        gB =       ( wk(i+1)-wk(i)  )
        gC = 0.5d0*( wk(i+1)-wk(i-1))
        if( gA*gB .le. 0 ) then
           grad = 0.d0
        else
           if( gA .gt. 0 ) then
              grad = min(2*gA,2*gB,gC)
           else
              grad = max(2*gA,2*gB,gC)
           endif
        endif
        wR(i-1) = wk(i) - 0.5d0 * grad
        wL(i)   = wk(i) + 0.5d0 * grad
     enddo
     do i=ix,ix
        wR(i-1) = wk(i)
     enddo
     
  else
     write(6,*) 'unknown limiter type'
     stop
     
  endif
  
  return
end subroutine limiter
