subroutine model(Ud,Um,Ue,Ub,margin,x,dx,ix)
  implicit none
  real(8) :: Ud(ix), Um(ix,3), Ue(ix), Ub(ix,3)
  real(8) :: x(ix), dx
  integer :: ix, margin
  integer :: i, izero
  real(8) :: ro, ux, uy, uz, u0, pr, bx, by, bz
  real(8) :: b2, bu
  real(8), parameter :: roL = 0.1d0
  real(8), parameter :: prL = 10.d0
  real(8), parameter :: uxL = 0.d0, uyL = 0.d0, uzL = sqrt(48.d0)
  real(8), parameter :: byL = 0.d0, bzL = 0.d0
  real(8), parameter :: roR = 1.d0
  real(8), parameter :: prR = 1.d0
  real(8), parameter :: uxR = 0.d0, uyR = 0.d0, uzR = 0.d0
  real(8), parameter :: byR = 0.d0, bzR = 0.d0

! grid
  dx=0.4d0/real(ix-margin*2)
  izero=ix/2
  x(izero)=-dx/2
  do i=izero+1,ix
     x(i) = x(i-1)+dx
  enddo
  do i=izero-1,1,-1
     x(i) = x(i+1)-dx
  enddo

! initial condition
  write(6,*) '[Left]'
  write(6,999) roL, prL, uzL, byL, bzL
  write(6,*) '[Right]'
  write(6,999) roR, prR, uzR, byR, bzR
999 format ('ro:', 1p, e10.3, ' pr:', 1p, e10.3, ' uz:', 1p, e10.3, &
         ' by:', 1p, e10.3, ' bz:', 1p, e10.3 )

  do i=1,ix
!    left
     if ( x(i) .lt. 0.d0 ) then
        ro = roL
        pr = prL
        ux = uxL
        uy = uyL
        uz = uzL
        by = byL
        bz = bzL
!     right
     else
        ro = roR
        pr = prR
        ux = uxR
        uy = uyR
        uz = uzR
        by = byR
        bz = bzR
     endif

     bx = 0
     b2 = bx*bx + by*by + bz*bz
     bu = bx*ux + by*uy + bz*uz
     u0 = sqrt(1.d0+ux**2+uy**2+uz**2)
     Ud(i)   = u0 * ro
     Um(i,1) = u0 * ( ro + 4.d0 * pr ) * ux + (b2*ux - bu*bx)/u0
     Um(i,2) = u0 * ( ro + 4.d0 * pr ) * uy + (b2*uy - bu*by)/u0
     Um(i,3) = u0 * ( ro + 4.d0 * pr ) * uz + (b2*uz - bu*bz)/u0
     Ue(i)   = u0*u0*(ro+4*pr)-pr + b2 - 0.5d0*(b2+bu**2)/(u0**2)
     Ub(i,1) = bx
     Ub(i,2) = by
     Ub(i,3) = bz
  enddo
     
  return
end subroutine model
